@@ -5,18 +5,17 @@ use pepper::{
     buffer_position::{BufferPosition, BufferRange},
     client,
     cursor::Cursor,
-    editor::Editor,
+    editor::EditorContext,
     editor_utils::MessageKind,
     glob::Glob,
     mode::ModeKind,
     picker::Picker,
-    platform::Platform,
     plugin::PluginHandle,
     word_database::{WordIndicesIter, WordKind},
 };
 
 use crate::{
-    client::{util, Client, ClientOperation, RequestState, ServerCapabilities},
+    client::{util, Client, RequestState, ServerCapabilities},
     json::{
         FromJson, Json, JsonArray, JsonConvertError, JsonInteger, JsonObject, JsonString, JsonValue,
     },
@@ -30,8 +29,7 @@ use crate::{
 
 pub(crate) fn on_request(
     client: &mut Client,
-    editor: &mut Editor,
-    clients: &mut pepper::client::ClientManager,
+    ctx: &mut EditorContext,
     request: ServerRequest,
 ) -> Result<JsonValue, ProtocolError> {
     client.write_to_log_file(|buf, json| {
@@ -150,7 +148,7 @@ pub(crate) fn on_request(
             }
 
             let (kind, message) = parse_params(request.params, &client.json)?;
-            editor.status_bar.write(kind).str(message);
+            ctx.editor.status_bar.write(kind).str(message);
             Ok(JsonValue::Null)
         }
         "window/showDocument" => {
@@ -185,8 +183,8 @@ pub(crate) fn on_request(
 
             let success = if let Some(true) = params.external {
                 false
-            } else if let Some(client_handle) = clients.focused_client() {
-                match editor.buffer_view_handle_from_path(
+            } else if let Some(client_handle) = ctx.clients.focused_client() {
+                match ctx.editor.buffer_view_handle_from_path(
                     client_handle,
                     path,
                     BufferProperties::text(),
@@ -194,14 +192,14 @@ pub(crate) fn on_request(
                 ) {
                     Ok(buffer_view_handle) => {
                         if let Some(true) = params.take_focus {
-                            let client = clients.get_mut(client_handle);
+                            let client = ctx.clients.get_mut(client_handle);
                             client.set_buffer_view_handle(
                                 Some(buffer_view_handle),
-                                &editor.buffer_views,
+                                &ctx.editor.buffer_views,
                             );
                         }
                         if let Some(range) = params.selection {
-                            let buffer_view = editor.buffer_views.get_mut(buffer_view_handle);
+                            let buffer_view = ctx.editor.buffer_views.get_mut(buffer_view_handle);
                             let mut cursors = buffer_view.cursors.mut_guard();
                             cursors.clear();
                             cursors.add(Cursor {
@@ -212,7 +210,7 @@ pub(crate) fn on_request(
                         true
                     }
                     Err(error) => {
-                        editor
+                        ctx.editor
                             .status_bar
                             .write(MessageKind::Error)
                             .fmt(format_args!("{}", error));
@@ -233,7 +231,7 @@ pub(crate) fn on_request(
 
 pub(crate) fn on_notification(
     client: &mut Client,
-    editor: &mut Editor,
+    ctx: &mut EditorContext,
     plugin_handle: PluginHandle,
     notification: ServerNotification,
 ) -> Result<(), ProtocolError> {
@@ -260,16 +258,18 @@ pub(crate) fn on_notification(
             }
             let message = message.as_str(&client.json);
             match message_type {
-                1 => editor.status_bar.write(MessageKind::Error).str(message),
-                2 => editor
+                1 => ctx.editor.status_bar.write(MessageKind::Error).str(message),
+                2 => ctx
+                    .editor
                     .status_bar
                     .write(MessageKind::Info)
                     .fmt(format_args!("warning: {}", message)),
-                3 => editor
+                3 => ctx
+                    .editor
                     .status_bar
                     .write(MessageKind::Info)
                     .fmt(format_args!("info: {}", message)),
-                4 => editor.status_bar.write(MessageKind::Info).str(message),
+                4 => ctx.editor.status_bar.write(MessageKind::Info).str(message),
                 _ => (),
             }
             Ok(())
@@ -302,9 +302,9 @@ pub(crate) fn on_notification(
             let Uri::Path(path) = Uri::parse(&client.root, uri)?;
 
             let mut buffer_handle = None;
-            for buffer in editor.buffers.iter() {
+            for buffer in ctx.editor.buffers.iter() {
                 if util::is_editor_path_equals_to_lsp_path(
-                    &editor.current_directory,
+                    &ctx.editor.current_directory,
                     &buffer.path,
                     &client.root,
                     path,
@@ -314,7 +314,8 @@ pub(crate) fn on_notification(
                 }
             }
             if let Some(buffer_handle) = buffer_handle {
-                let mut lints = editor
+                let mut lints = ctx
+                    .editor
                     .buffers
                     .get_mut(buffer_handle)
                     .lints
@@ -343,15 +344,13 @@ pub(crate) fn on_notification(
 
 pub(crate) fn on_response(
     client: &mut Client,
-    editor: &mut Editor,
-    platform: &mut Platform,
-    clients: &mut pepper::client::ClientManager,
+    ctx: &mut EditorContext,
     plugin_handle: PluginHandle,
     response: ServerResponse,
-) -> Result<ClientOperation, ProtocolError> {
+) -> Result<(), ProtocolError> {
     let method = match client.pending_requests.take(response.id) {
         Some(method) => method,
-        None => return Ok(ClientOperation::None),
+        None => return Ok(()),
     };
 
     client.write_to_log_file(|buf, json| {
@@ -382,8 +381,8 @@ pub(crate) fn on_response(
         Ok(result) => result,
         Err(error) => {
             client.request_state = RequestState::Idle;
-            util::write_response_error(&mut editor.status_bar, error, &client.json);
-            return Ok(ClientOperation::None);
+            util::write_response_error(&mut ctx.editor.status_bar, error, &client.json);
+            return Ok(());
         }
     };
 
@@ -406,30 +405,32 @@ pub(crate) fn on_response(
             }
 
             match server_name {
-                "" => editor
+                "" => ctx
+                    .editor
                     .status_bar
                     .write(MessageKind::Info)
                     .str("lsp server started"),
-                _ => editor
+                _ => ctx
+                    .editor
                     .status_bar
                     .write(MessageKind::Info)
                     .fmt(format_args!("lsp server '{}' started", server_name)),
             }
 
             client.initialized = true;
-            client.notify(platform, "initialized", JsonObject::default());
+            client.notify(&mut ctx.platform, "initialized", JsonObject::default());
 
-            for buffer in editor.buffers.iter() {
-                util::send_did_open(client, editor, platform, buffer.handle());
+            for buffer in ctx.editor.buffers.iter() {
+                util::send_did_open(client, &ctx.editor, &mut ctx.platform, buffer.handle());
             }
 
-            Ok(ClientOperation::None)
+            Ok(())
         }
         "textDocument/hover" => {
             let contents = result.get("contents", &client.json);
             let info = util::extract_markup_content(contents, &client.json);
-            editor.status_bar.write(MessageKind::Info).str(info);
-            Ok(ClientOperation::None)
+            ctx.editor.status_bar.write(MessageKind::Info).str(info);
+            Ok(())
         }
         "textDocument/signatureHelp" => {
             #[derive(Default)]
@@ -489,63 +490,42 @@ pub(crate) fn on_response(
                     .nth(sh.active_signature)
             }) {
                 Some(signature) => signature,
-                None => return Ok(ClientOperation::None),
+                None => return Ok(()),
             };
             let signature = SignatureInformation::from_json(signature, &client.json)?;
             let label = signature.label.as_str(&client.json);
 
             if signature.documentation.is_empty() {
-                editor.status_bar.write(MessageKind::Info).str(label);
+                ctx.editor.status_bar.write(MessageKind::Info).str(label);
             } else {
-                editor
+                ctx.editor
                     .status_bar
                     .write(MessageKind::Info)
                     .fmt(format_args!("{}\n{}", signature.documentation, label));
             }
 
-            Ok(ClientOperation::None)
+            Ok(())
         }
         "textDocument/definition" => {
             let client_handle = match client.request_state {
                 RequestState::Definition { client_handle } => client_handle,
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
-            goto_definition(
-                client,
-                editor,
-                clients,
-                plugin_handle,
-                client_handle,
-                result,
-            )
+            goto_definition(client, ctx, plugin_handle, client_handle, result)
         }
         "textDocument/declaration" => {
             let client_handle = match client.request_state {
                 RequestState::Declaration { client_handle } => client_handle,
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
-            goto_definition(
-                client,
-                editor,
-                clients,
-                plugin_handle,
-                client_handle,
-                result,
-            )
+            goto_definition(client, ctx, plugin_handle, client_handle, result)
         }
         "textDocument/implementation" => {
             let client_handle = match client.request_state {
                 RequestState::Implementation { client_handle } => client_handle,
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
-            goto_definition(
-                client,
-                editor,
-                clients,
-                plugin_handle,
-                client_handle,
-                result,
-            )
+            goto_definition(client, ctx, plugin_handle, client_handle, result)
         }
         "textDocument/references" => {
             let (client_handle, context_len) = match client.request_state {
@@ -553,23 +533,24 @@ pub(crate) fn on_response(
                     client_handle,
                     context_len,
                 } => (client_handle, context_len),
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
             client.request_state = RequestState::Idle;
             let locations = match result {
                 JsonValue::Array(locations) => locations,
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
 
-            let mut buffer_name = editor.string_pool.acquire();
+            let mut buffer_name = ctx.editor.string_pool.acquire();
             for location in locations.clone().elements(&client.json) {
                 let location = DocumentLocation::from_json(location, &client.json)?;
                 let Uri::Path(path) = Uri::parse(&client.root, location.uri.as_str(&client.json))?;
 
-                if let Some(buffer) = editor
+                if let Some(buffer) = ctx
+                    .editor
                     .buffers
-                    .find_with_path(&editor.current_directory, path)
-                    .map(|h| editor.buffers.get(h))
+                    .find_with_path(&ctx.editor.current_directory, path)
+                    .map(|h| ctx.editor.buffers.get(h))
                 {
                     let range = location.range.into_buffer_range();
                     for text in buffer.content().text_range(range) {
@@ -583,35 +564,36 @@ pub(crate) fn on_response(
             }
             buffer_name.push_str(".refs");
 
-            let buffer_view_handle = editor.buffer_view_handle_from_path(
+            let buffer_view_handle = ctx.editor.buffer_view_handle_from_path(
                 client_handle,
                 Path::new(&buffer_name),
                 BufferProperties::scratch(),
                 true,
             );
-            editor.string_pool.release(buffer_name);
+            ctx.editor.string_pool.release(buffer_name);
             let buffer_view_handle = match buffer_view_handle {
                 Ok(handle) => handle,
                 Err(error) => {
-                    editor
+                    ctx.editor
                         .status_bar
                         .write(MessageKind::Error)
                         .fmt(format_args!("{}", error));
-                    return Ok(ClientOperation::None);
+                    return Ok(());
                 }
             };
 
             let mut count = 0;
             let mut context_buffer = BufferContent::new();
 
-            let buffer_view = editor.buffer_views.get(buffer_view_handle);
-            let buffer = editor.buffers.get_mut(buffer_view.buffer_handle);
+            let buffer_view = ctx.editor.buffer_views.get(buffer_view_handle);
+            let buffer = ctx.editor.buffers.get_mut(buffer_view.buffer_handle);
+
+            let range = BufferRange::between(BufferPosition::zero(), buffer.content().end());
+            buffer.delete_range(&mut ctx.editor.word_database, range, &mut ctx.editor.events);
 
             buffer.properties = BufferProperties::scratch();
-            let range = BufferRange::between(BufferPosition::zero(), buffer.content().end());
-            buffer.delete_range(&mut editor.word_database, range, &mut editor.events);
 
-            let mut text = editor.string_pool.acquire();
+            let mut text = ctx.editor.string_pool.acquire();
             let mut last_path = "";
             for location in locations.elements(&client.json) {
                 let location = match DocumentLocation::from_json(location, &client.json) {
@@ -663,10 +645,10 @@ pub(crate) fn on_response(
 
                 let position = buffer.content().end();
                 buffer.insert_text(
-                    &mut editor.word_database,
+                    &mut ctx.editor.word_database,
                     position,
                     &text,
-                    &mut editor.events,
+                    &mut ctx.editor.events,
                 );
                 text.clear();
 
@@ -682,17 +664,18 @@ pub(crate) fn on_response(
             }
 
             buffer.insert_text(
-                &mut editor.word_database,
+                &mut ctx.editor.word_database,
                 BufferPosition::zero(),
                 &text,
-                &mut editor.events,
+                &mut ctx.editor.events,
             );
-            editor.string_pool.release(text);
+            ctx.editor.string_pool.release(text);
 
-            let client = clients.get_mut(client_handle);
-            client.set_buffer_view_handle(Some(buffer_view_handle), &editor.buffer_views);
+            let client = ctx.clients.get_mut(client_handle);
+            client.set_buffer_view_handle(Some(buffer_view_handle), &ctx.editor.buffer_views);
 
-            let mut cursors = editor
+            let mut cursors = ctx
+                .editor
                 .buffer_views
                 .get_mut(buffer_view_handle)
                 .cursors
@@ -703,7 +686,7 @@ pub(crate) fn on_response(
                 position: BufferPosition::zero(),
             });
 
-            Ok(ClientOperation::None)
+            Ok(())
         }
         "textDocument/prepareRename" => {
             let (buffer_handle, buffer_position) = match client.request_state {
@@ -711,19 +694,19 @@ pub(crate) fn on_response(
                     buffer_handle,
                     buffer_position,
                 } => (buffer_handle, buffer_position),
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
             client.request_state = RequestState::Idle;
             let result = match result {
                 JsonValue::Null => {
-                    editor
+                    ctx.editor
                         .status_bar
                         .write(MessageKind::Error)
                         .str("could not rename item under cursor");
-                    return Ok(ClientOperation::None);
+                    return Ok(());
                 }
                 JsonValue::Object(result) => result,
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
             let mut range = DocumentRange::default();
             let mut placeholder: Option<JsonString> = None;
@@ -741,7 +724,7 @@ pub(crate) fn on_response(
                 }
             }
 
-            let buffer = editor.buffers.get(buffer_handle);
+            let buffer = ctx.editor.buffers.get(buffer_handle);
 
             let mut range = range.into_buffer_range();
             if let Some(true) = default_behaviour {
@@ -749,7 +732,7 @@ pub(crate) fn on_response(
                 range = BufferRange::between(word.position, word.end_position());
             }
 
-            let mut input = editor.string_pool.acquire();
+            let mut input = ctx.editor.string_pool.acquire();
             match placeholder {
                 Some(text) => input.push_str(text.as_str(&client.json)),
                 None => {
@@ -759,44 +742,50 @@ pub(crate) fn on_response(
                 }
             }
 
-            let op = read_line::enter_rename_mode(editor, plugin_handle, &input);
-            editor.string_pool.release(input);
+            read_line::enter_rename_mode(ctx, plugin_handle, &input, client);
+
+            ctx.editor.string_pool.release(input);
 
             client.request_state = RequestState::FinishRename {
                 buffer_handle,
                 buffer_position,
             };
-            Ok(op)
+            Ok(())
         }
         "textDocument/rename" => {
             let edit = WorkspaceEdit::from_json(result, &client.json)?;
-            edit.apply(editor, &mut client.temp_edits, &client.root, &client.json);
-            Ok(ClientOperation::None)
+            edit.apply(
+                &mut ctx.editor,
+                &mut client.temp_edits,
+                &client.root,
+                &client.json,
+            );
+            Ok(())
         }
         "textDocument/codeAction" => {
             match client.request_state {
                 RequestState::CodeAction => (),
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
             client.request_state = RequestState::Idle;
             let actions = match result {
                 JsonValue::Array(actions) => actions,
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
 
-            editor.picker.clear();
+            ctx.editor.picker.clear();
             for action in actions
                 .clone()
                 .elements(&client.json)
                 .filter_map(|a| DocumentCodeAction::from_json(a, &client.json).ok())
                 .filter(|a| !a.disabled)
             {
-                editor
+                ctx.editor
                     .picker
                     .add_custom_entry(action.title.as_str(&client.json));
             }
 
-            let op = picker::enter_code_action_mode(editor, plugin_handle, client);
+            picker::enter_code_action_mode(ctx, plugin_handle, client);
 
             client.request_state = RequestState::FinishCodeAction;
             client.request_raw_json.clear();
@@ -804,17 +793,17 @@ pub(crate) fn on_response(
                 .json
                 .write(&mut client.request_raw_json, &actions.into());
 
-            Ok(op)
+            Ok(())
         }
         "textDocument/documentSymbol" => {
             let buffer_view_handle = match client.request_state {
                 RequestState::DocumentSymbols { buffer_view_handle } => buffer_view_handle,
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
             client.request_state = RequestState::Idle;
             let symbols = match result {
                 JsonValue::Array(symbols) => symbols,
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
 
             fn add_symbols(picker: &mut Picker, depth: usize, symbols: JsonArray, json: &Json) {
@@ -844,10 +833,10 @@ pub(crate) fn on_response(
                 }
             }
 
-            editor.picker.clear();
-            add_symbols(&mut editor.picker, 0, symbols.clone(), &client.json);
+            ctx.editor.picker.clear();
+            add_symbols(&mut ctx.editor.picker, 0, symbols.clone(), &client.json);
 
-            let op = picker::enter_document_symbol_mode(editor, plugin_handle, client);
+            picker::enter_document_symbol_mode(ctx, plugin_handle, client);
 
             client.request_state = RequestState::FinishDocumentSymbols { buffer_view_handle };
             client.request_raw_json.clear();
@@ -855,20 +844,20 @@ pub(crate) fn on_response(
                 .json
                 .write(&mut client.request_raw_json, &symbols.into());
 
-            Ok(op)
+            Ok(())
         }
         "workspace/symbol" => {
             match client.request_state {
                 RequestState::WorkspaceSymbols => (),
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
             client.request_state = RequestState::Idle;
             let symbols = match result {
                 JsonValue::Array(symbols) => symbols,
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
 
-            editor.picker.clear();
+            ctx.editor.picker.clear();
             for symbol in symbols
                 .clone()
                 .elements(&client.json)
@@ -878,15 +867,15 @@ pub(crate) fn on_response(
                 match symbol.container_name {
                     Some(container_name) => {
                         let container_name = container_name.as_str(&client.json);
-                        editor
+                        ctx.editor
                             .picker
                             .add_custom_entry_fmt(format_args!("{} ({})", name, container_name,));
                     }
-                    None => editor.picker.add_custom_entry(name),
+                    None => ctx.editor.picker.add_custom_entry(name),
                 }
             }
 
-            let op = picker::enter_workspace_symbol_mode(editor, plugin_handle, client);
+            picker::enter_workspace_symbol_mode(ctx, plugin_handle, client);
 
             client.request_state = RequestState::FinishWorkspaceSymbols;
             client.request_raw_json.clear();
@@ -894,27 +883,42 @@ pub(crate) fn on_response(
                 .json
                 .write(&mut client.request_raw_json, &symbols.into());
 
-            Ok(op)
+            Ok(())
         }
         "textDocument/formatting" => {
-            let buffer_handle = match client.request_state {
-                RequestState::Formatting { buffer_handle } => buffer_handle,
-                _ => return Ok(ClientOperation::None),
+            let buffer_view_handle = match client.request_state {
+                RequestState::Formatting { buffer_view_handle } => buffer_view_handle,
+                _ => return Ok(()),
             };
             client.request_state = RequestState::Idle;
             let edits = match result {
                 JsonValue::Array(edits) => edits,
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
+
+            let buffer_view = ctx.editor.buffer_views.get(buffer_view_handle);
+            let buffer_handle = buffer_view.buffer_handle;
+            let position = buffer_view.cursors.main_cursor().position;
+
             TextEdit::apply_edits(
-                editor,
+                &mut ctx.editor,
                 buffer_handle,
                 &mut client.temp_edits,
                 edits,
                 &client.json,
             );
 
-            Ok(ClientOperation::None)
+            ctx.trigger_event_handlers();
+
+            let buffer_view = ctx.editor.buffer_views.get_mut(buffer_view_handle);
+            let mut cursors = buffer_view.cursors.mut_guard();
+            cursors.clear();
+            cursors.add(Cursor {
+                anchor: position,
+                position,
+            });
+
+            Ok(())
         }
         "textDocument/completion" => {
             let (client_handle, buffer_handle) = match client.request_state {
@@ -922,39 +926,39 @@ pub(crate) fn on_response(
                     client_handle,
                     buffer_handle,
                 } => (client_handle, buffer_handle),
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
             client.request_state = RequestState::Idle;
 
-            if editor.mode.kind() != ModeKind::Insert {
-                return Ok(ClientOperation::None);
+            if ctx.editor.mode.kind() != ModeKind::Insert {
+                return Ok(());
             }
 
-            let buffer_view_handle = match clients.get(client_handle).buffer_view_handle() {
+            let buffer_view_handle = match ctx.clients.get(client_handle).buffer_view_handle() {
                 Some(handle) => handle,
-                None => return Ok(ClientOperation::None),
+                None => return Ok(()),
             };
-            let buffer_view = editor.buffer_views.get(buffer_view_handle);
+            let buffer_view = ctx.editor.buffer_views.get(buffer_view_handle);
             if buffer_view.buffer_handle != buffer_handle {
-                return Ok(ClientOperation::None);
+                return Ok(());
             }
-            let buffer = editor.buffers.get(buffer_handle).content();
+            let buffer = ctx.editor.buffers.get(buffer_handle).content();
 
             let completions = match result {
                 JsonValue::Array(completions) => completions,
                 JsonValue::Object(completions) => match completions.get("items", &client.json) {
                     JsonValue::Array(completions) => completions,
-                    _ => return Ok(ClientOperation::None),
+                    _ => return Ok(()),
                 },
-                _ => return Ok(ClientOperation::None),
+                _ => return Ok(()),
             };
 
-            editor.picker.clear();
+            ctx.editor.picker.clear();
             for completion in completions.elements(&client.json) {
                 if let Ok(completion) = DocumentCompletionItem::from_json(completion, &client.json)
                 {
                     let text = completion.text.as_str(&client.json);
-                    editor.picker.add_custom_entry(text);
+                    ctx.editor.picker.add_custom_entry(text);
                 }
             }
 
@@ -965,22 +969,21 @@ pub(crate) fn on_response(
                 WordKind::Identifier => word.text,
                 _ => "",
             };
-            editor.picker.filter(WordIndicesIter::empty(), filter);
+            ctx.editor.picker.filter(WordIndicesIter::empty(), filter);
 
-            Ok(ClientOperation::None)
+            Ok(())
         }
-        _ => Ok(ClientOperation::None),
+        _ => Ok(()),
     }
 }
 
 fn goto_definition(
     client: &mut Client,
-    editor: &mut Editor,
-    clients: &mut pepper::client::ClientManager,
+    ctx: &mut EditorContext,
     plugin_handle: PluginHandle,
     client_handle: client::ClientHandle,
     result: JsonValue,
-) -> Result<ClientOperation, ProtocolError> {
+) -> Result<(), ProtocolError> {
     enum DefinitionLocation {
         Single(DocumentLocation),
         Many(JsonArray),
@@ -1017,17 +1020,18 @@ fn goto_definition(
         DefinitionLocation::Single(location) => {
             let Uri::Path(path) = Uri::parse(&client.root, location.uri.as_str(&client.json))?;
 
-            match editor.buffer_view_handle_from_path(
+            match ctx.editor.buffer_view_handle_from_path(
                 client_handle,
                 path,
                 BufferProperties::text(),
                 false,
             ) {
                 Ok(buffer_view_handle) => {
-                    let client = clients.get_mut(client_handle);
-                    client.set_buffer_view_handle(Some(buffer_view_handle), &editor.buffer_views);
+                    let client = ctx.clients.get_mut(client_handle);
+                    client
+                        .set_buffer_view_handle(Some(buffer_view_handle), &ctx.editor.buffer_views);
 
-                    let buffer_view = editor.buffer_views.get_mut(buffer_view_handle);
+                    let buffer_view = ctx.editor.buffer_views.get_mut(buffer_view_handle);
                     let position = location.range.start.into_buffer_position();
                     let mut cursors = buffer_view.cursors.mut_guard();
                     cursors.clear();
@@ -1036,16 +1040,17 @@ fn goto_definition(
                         position,
                     });
                 }
-                Err(error) => editor
+                Err(error) => ctx
+                    .editor
                     .status_bar
                     .write(MessageKind::Error)
                     .fmt(format_args!("{}", error)),
             }
 
-            Ok(ClientOperation::None)
+            Ok(())
         }
         DefinitionLocation::Many(locations) => {
-            editor.picker.clear();
+            ctx.editor.picker.clear();
             for location in locations
                 .elements(&client.json)
                 .filter_map(|l| DocumentLocation::from_json(l, &client.json).ok())
@@ -1060,7 +1065,7 @@ fn goto_definition(
                 };
 
                 let position = location.range.start.into_buffer_position();
-                editor.picker.add_custom_entry_fmt(format_args!(
+                ctx.editor.picker.add_custom_entry_fmt(format_args!(
                     "{}:{},{}",
                     path,
                     position.line_index + 1,
@@ -1068,9 +1073,10 @@ fn goto_definition(
                 ));
             }
 
-            let op = picker::enter_definition_mode(editor, plugin_handle);
-            Ok(op)
+            picker::enter_definition_mode(ctx, plugin_handle, client);
+            Ok(())
         }
-        DefinitionLocation::Invalid => Ok(ClientOperation::None),
+        DefinitionLocation::Invalid => Ok(()),
     }
 }
+
